% rossreturnmap.m
% program calculates return map in the radial coordinate using 
% fourth order Runge-Kutta scheme and "Henon's trick"

clc; close all; clear all;
format long e;

n=3;

y(1)=1.0;  % initial
y(2)=0.1;  % values
y(3)=0.7;
t=0.0;
ttot=25000;  % total time
ttr=5000;  % transient time
h=.001;  % time step !!if you change here, change in RK4 also!!
skip=1;  % used to ignore testing for return map condition
count=0;  % counts number of return map points

ang=(230/180)*pi;% defines theta coordinate of the section 
tan_c=tan(ang);  % don't use if ang~pi/2 or ~3pi/2 
                 % instead use cot_c
%cot_c=cot(ang); % don't use if ang~0 or ~pi, use tan_c instead
                 
%datafile1=('ross.txt');       % file for 3D flow in phase space
%rossdata=fopen(datafile1,'w');
datafile2=('ret230.txt');         % stores radial value upon 
retdata=fopen(datafile2,'w');        % each crossing
datafile3=('r,z_ptsnearcrit230.txt'); % stores r,z vals near
zvaldata=fopen(datafile3,'w');           % the critical point 

while (t<ttr)  % let transients die out
    
    yout=RK4(y); % integration carried out in cartesian coordinates
    t=t+h;
    y=yout;
end

while (t<ttot)
    
    yout=RK4(y);
    
    if ((y(2)>=0)&(yout(2)<=0)) % !!change these conditions anytime
        skip=0;                 % you change 'ang'!! 
    end
            
    if ((yout(2)<=0)&(yout(1)<=0)&(skip==0)) % !!condition depends on which
                                 % quadrant the section (i.e., 'ang') is in!!
    %tcot=y(1)/y(2); % use to compare against cot_c
    %tcotout=yout(1)/yout(2);
    ttan=y(2)/y(1);  % use to compare against tan_c
    ttanout=yout(2)/yout(1);
    
        if ((ttan<tan_c)&(ttanout>tan_c))   % implies traj crossed  
        %if ((tcot>cot_c)&(tcotout<cot_c))  % the section
             yout=retRK4pol(y,t,ang);  % integrate using radial
                                       % coordinate and Henon's trick
             t=yout(2);   % switch t and theta back
             yout(2)=ang; % make theta the coord of the section
             count=count+1; % count number of returns
             rplot(count)=yout(1);
             fprintf(retdata, '%16.14g\n', yout(1));
             
             rad=yout(1);  % used to convert from cylindrical to cartesian
             thet=yout(2);
             
             %if ((5.95<yout(1))&(yout(1)<6.15)) %   stores r,z-values of 
             %    fprintf(zvaldata, '%16.14g %16.14g\n',... % pts near the
             %    yout(1),yout(3));                       % critical point
                                                          
             end
             skip=1;           
             yout(1)=rad*cos(thet); % convert back to cartesian coords
             yout(2)=rad*sin(thet);
             y=yout;
             
        elseif (ttan==tan_c) % if landed on the section on prev step
             rad=sqrt(y(1)^2+y(2)^2); % don't advance time
             count=count+1;
             rplot(count)=rad;
             fprintf(retdata, '%16.14g\n', rad);
             skip=1;
             
        elseif (ttanout==tan_c) % if landed on the section on the current 
             rad=sqrt(yout(1)^2+yout(2)^2); % step must advance time 
             count=count+1;
             rplot(count)=rad;
             fprintf(retdata, '%16.14g\n', rad);
             y=yout;
             t=t+h;
             skip=1;
             
        else
             %fprintf(rossdata, '%16.14g %16.14g %16.14g\n',...
             %        yout(1), yout(2), yout(3));
             t=t+h;
             y=yout;
        end
    
    else
        %fprintf(rossdata, '%16.14g %16.14g %16.14g\n',...
        %        yout(1), yout(2), yout(3));
        t=t+h;
        y=yout;
    end
    
end

%fclose(rossdata);
fclose(retdata);
fclose(zvaldata);
%rossflowplot;  % used to plot 3D flow (if needed)

for w=1:count-1            % creates array for return map plot
    data22(w,1)=rplot(w);  
    data22(w,2)=rplot(w+1);
end

%h2=figure(2);
plot(data22(:,1),data22(:,2),'k.') % plots R_(n+1) vs R_(n)
grid on;
axis on;
xlabel('R(n)');
ylabel('R(n+1)');